@@ -1,5 +1,5 @@
-selgmented <-function(olm, seg.Z, Kmax=2, type=c("score", "bic", "davies", "aic"), alpha=0.05, 
-                  control=seg.control(), refit=FALSE, stop.if=5, return.fit=TRUE, bonferroni=FALSE, #improve.after.G=FALSE,
+selgmented <-function(olm, seg.Z, Kmax=10, type=c("bic", "aic", "score", "davies"), alpha=0.05, 
+                  control=seg.control(), refit=FALSE, stop.if=6, return.fit=TRUE, bonferroni=FALSE, #improve.after.G=FALSE,
                   msg=TRUE, plot.ic=FALSE, th=NULL, G=1, check.dslope=TRUE){ #}, a=1){
   
   #vedere bene il fatto del th!!
